@@ -1,32 +1,7 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
@@ -40,9 +15,12 @@
 
 #include "Channel.h"
 #include "Database.h"
-#include "Global.h"
 #include "ServerHandler.h"
 #include "WebFetch.h"
+#include "ServerResolver.h"
+
+// We define a global macro called 'g'. This can lead to issues when included code uses 'g' as a type or parameter name (like protobuf 3.7 does). As such, for now, we have to make this our last include.
+#include "Global.h"
 
 QMap<QString, QIcon> ServerItem::qmIcons;
 QList<PublicInfo> ConnectDialog::qlPublicServers;
@@ -78,6 +56,22 @@ void PingStats::reset() {
 	init();
 }
 
+ServerViewDelegate::ServerViewDelegate(QObject *p) : QStyledItemDelegate(p) {
+}
+
+ServerViewDelegate::~ServerViewDelegate() {
+}
+
+void ServerViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const {
+	// Allow a ServerItem's BackgroundRole to override the current theme's default color.
+	QVariant bg = index.data(Qt::BackgroundRole);
+	if (bg.isValid()) {
+		painter->fillRect(option.rect, bg.value<QBrush>());
+	}
+
+	QStyledItemDelegate::paint(painter, option, index);
+}
+
 ServerView::ServerView(QWidget *p) : QTreeWidget(p) {
 	siFavorite = new ServerItem(tr("Favorite"), ServerItem::FavoriteType);
 	addTopLevelItem(siFavorite);
@@ -97,10 +91,12 @@ ServerView::ServerView(QWidget *p) : QTreeWidget(p) {
 		siPublic = new ServerItem(tr("Public Internet"), ServerItem::PublicType);
 		siPublic->setChildIndicatorPolicy(QTreeWidgetItem::ShowIndicator);
 		addTopLevelItem(siPublic);
-		
-		
+
 		siPublic->setExpanded(false);
-	
+
+		// The continent code is empty when the server's IP address is not in the GeoIP database
+		qmContinentNames.insert(QLatin1String(""), tr("Unknown"));
+
 		qmContinentNames.insert(QLatin1String("af"), tr("Africa"));
 		qmContinentNames.insert(QLatin1String("as"), tr("Asia"));
 		qmContinentNames.insert(QLatin1String("na"), tr("North America"));
@@ -109,7 +105,7 @@ ServerView::ServerView(QWidget *p) : QTreeWidget(p) {
 		qmContinentNames.insert(QLatin1String("oc"), tr("Oceania"));
 	} else {
 		qWarning()<< "Public list disabled";
-		
+
 		siPublic = NULL;
 	}
 }
@@ -120,7 +116,7 @@ ServerView::~ServerView() {
 	delete siPublic;
 }
 
-QMimeData *ServerView::mimeData(const QList<QTreeWidgetItem *>& mimeitems) const {
+QMimeData *ServerView::mimeData(const QList<QTreeWidgetItem *> mimeitems) const {
 	if (mimeitems.isEmpty())
 		return NULL;
 
@@ -190,7 +186,7 @@ bool ServerView::dropMimeData(QTreeWidgetItem *, int, const QMimeData *mime, Qt:
 
 ServerItem *ServerView::getParent(const QString &continentcode, const QString &countrycode, const QString &countryname, const QString &usercontinent, const QString &usercountry) {
 	ServerItem *continent = qmContinent.value(continentcode);
-	if (! continent) {
+	if (!continent) {
 		QString name = qmContinentNames.value(continentcode);
 		if (name.isEmpty())
 			name = continentcode;
@@ -198,26 +194,32 @@ ServerItem *ServerView::getParent(const QString &continentcode, const QString &c
 		qmContinent.insert(continentcode, continent);
 		siPublic->addServerItem(continent);
 
-		if (continentcode == usercontinent) {
-			continent->setExpanded(true);
-			scrollToItem(continent, QAbstractItemView::PositionAtTop);
+		if (!continentcode.isEmpty()) {
+			if (continentcode == usercontinent) {
+				continent->setExpanded(true);
+				scrollToItem(continent, QAbstractItemView::PositionAtTop);
+			}
 		} else {
-			continent->setExpanded(false);
+			continent->setChildIndicatorPolicy(QTreeWidgetItem::ShowIndicator);
 		}
 	}
+
+	// If the continent code is empty, we put the server directly into the "Unknown" continent
+	if (continentcode.isEmpty()) {
+		return continent;
+	}
+
 	ServerItem *country = qmCountry.value(countrycode);
-	if (! country) {
+	if (!country) {
 		country = new ServerItem(countryname, ServerItem::PublicType, continentcode, countrycode);
 		qmCountry.insert(countrycode, country);
 		country->setChildIndicatorPolicy(QTreeWidgetItem::ShowIndicator);
 
 		continent->addServerItem(country);
 
-		if (countrycode == usercountry) {
+		if (!countrycode.isEmpty() && countrycode == usercountry) {
 			country->setExpanded(true);
 			scrollToItem(country, QAbstractItemView::PositionAtTop);
-		} else {
-			country->setExpanded(false);
 		}
 	}
 	return country;
@@ -364,7 +366,7 @@ ServerItem::~ServerItem() {
 		delete si;
 }
 
-ServerItem *ServerItem::fromMimeData(const QMimeData *mime, QWidget *p) {
+ServerItem *ServerItem::fromMimeData(const QMimeData *mime, bool default_name, QWidget *p, bool convertHttpUrls) {
 	if (mime->hasFormat(QLatin1String("OriginatedInMumble")))
 		return NULL;
 
@@ -377,20 +379,65 @@ ServerItem *ServerItem::fromMimeData(const QMimeData *mime, QWidget *p) {
 	QString qsFile = url.toLocalFile();
 	if (! qsFile.isEmpty()) {
 		QFile f(qsFile);
-		if (f.open(QIODevice::ReadOnly)) {
+		// Make sure we don't accidently read something big the user
+		// happened to have in his clipboard. We only want to look
+		// at small link files.
+		if (f.open(QIODevice::ReadOnly) && f.size() < 10240) {
 			QByteArray qba = f.readAll();
 			f.close();
 
 			url = QUrl::fromEncoded(qba, QUrl::StrictMode);
 			if (! url.isValid()) {
+				// Windows internet shortcut files (.url) are an ini with an URL value
 				QSettings qs(qsFile, QSettings::IniFormat);
 				url = QUrl::fromEncoded(qs.value(QLatin1String("InternetShortcut/URL")).toByteArray(), QUrl::StrictMode);
 			}
 		}
 	}
 
-	if (! url.isValid() || (url.scheme() != QLatin1String("mumble")))
+	if (default_name) {
+#if QT_VERSION >= 0x050000
+		QUrlQuery query(url);
+		if (! query.hasQueryItem(QLatin1String("title"))) {
+			query.addQueryItem(QLatin1String("title"), url.host());
+		}
+#else
+		if (! url.hasQueryItem(QLatin1String("title"))) {
+			url.addQueryItem(QLatin1String("title"), url.host());
+		}
+#endif
+	}
+
+	if (! url.isValid()) {
 		return NULL;
+	}
+
+	// An URL from text without a scheme will have the hostname text
+	// in the QUrl scheme and no hostname. We do not want to use that.
+	if (url.host().isEmpty()) {
+		return NULL;
+	}
+
+	// Some communication programs automatically create http links from domains.
+	// When a user sends another user a domain to connect to, and http is added wrongly,
+	// we do our best to remove it again.
+	if (convertHttpUrls && (
+	    url.scheme() == QLatin1String("http")
+	    || url.scheme() == QLatin1String("https"))) {
+		url.setScheme(QLatin1String("mumble"));
+	}
+
+	return fromUrl(url, p);
+}
+
+ServerItem *ServerItem::fromUrl(QUrl url, QWidget *p) {
+	if (! url.isValid() || (url.scheme() != QLatin1String("mumble"))) {
+		return NULL;
+	}
+
+#if QT_VERSION >= 0x050000
+	QUrlQuery query(url);
+#endif
 
 	if (url.userName().isEmpty()) {
 		if (g.s.qsUsername.isEmpty()) {
@@ -405,13 +452,17 @@ ServerItem *ServerItem::fromMimeData(const QMimeData *mime, QWidget *p) {
 		url.setUserName(g.s.qsUsername);
 	}
 
-	if (! url.hasQueryItem(QLatin1String("title")))
-		url.addQueryItem(QLatin1String("title"), url.host());
+#if QT_VERSION >= 0x050000
+	ServerItem *si = new ServerItem(query.queryItemValue(QLatin1String("title")), url.host(), static_cast<unsigned short>(url.port(DEFAULT_MUMBLE_PORT)), url.userName(), url.password());
 
+	if (query.hasQueryItem(QLatin1String("url")))
+		si->qsUrl = query.queryItemValue(QLatin1String("url"));
+#else
 	ServerItem *si = new ServerItem(url.queryItemValue(QLatin1String("title")), url.host(), static_cast<unsigned short>(url.port(DEFAULT_MUMBLE_PORT)), url.userName(), url.password());
 
 	if (url.hasQueryItem(QLatin1String("url")))
 		si->qsUrl = url.queryItemValue(QLatin1String("url"));
+#endif
 
 	return si;
 }
@@ -427,8 +478,13 @@ QVariant ServerItem::data(int column, int role) const {
 						return loadIcon(QLatin1String("skin:emblems/emblem-favorite.svg"));
 					else if (itType == LANType)
 						return loadIcon(QLatin1String("skin:places/network-workgroup.svg"));
-					else if (! qsCountryCode.isEmpty())
-						return loadIcon(QString::fromLatin1(":/flags/%1.png").arg(qsCountryCode));
+					else if (! qsCountryCode.isEmpty()) {
+						QString flag = QString::fromLatin1(":/flags/%1.svg").arg(qsCountryCode);
+						if (!QFileInfo(flag).exists()) {
+							flag = QLatin1String("skin:categories/applications-internet.svg");
+						}
+						return loadIcon(flag);
+					}
 					else
 						return loadIcon(QLatin1String("skin:categories/applications-internet.svg"));
 			}
@@ -441,12 +497,13 @@ QVariant ServerItem::data(int column, int role) const {
 				case 1:
 					return (dPing > 0.0) ? QString::number(uiPing) : QVariant();
 				case 2:
-					return uiUsers ? QString::fromLatin1("%1/%2").arg(uiUsers).arg(uiMaxUsers) : QVariant();
+					return uiUsers ? QString::fromLatin1("%1/%2 ").arg(uiUsers).arg(uiMaxUsers) : QVariant();
 			}
 		} else if (role == Qt::ToolTipRole) {
 			QStringList qsl;
-			foreach(const QHostAddress &qha, qlAddresses)
-				qsl << Qt::escape(qha.toString());
+			foreach(const ServerAddress &addr, qlAddresses) {
+				qsl << Qt::escape(addr.host.toString() + QLatin1String(":") + QString::number(static_cast<unsigned long>(addr.port)));
+			}
 
 			double ploss = 100.0;
 
@@ -570,10 +627,9 @@ FavoriteServer ServerItem::toFavoriteServer() const {
 }
 
 
-/*!
-  \fn QMimeData *ServerItem::toMimeData() const
-  This function turns a ServerItem object into a QMimeData object holding a URL to the server.
-*/
+/**
+ * This function turns a ServerItem object into a QMimeData object holding a URL to the server.
+ */
 QMimeData *ServerItem::toMimeData() const {
 	QMimeData *mime = ServerItem::toMimeData(qsName, qsHostname, usPort);
 
@@ -583,12 +639,11 @@ QMimeData *ServerItem::toMimeData() const {
 	return mime;
 }
 
-/*!
-  \fn QMimeData *ServerItem::toMimeData(const QString &name, const QString &host, unsigned short port, const QString &channel)
-  This function creates a QMimeData object containing a URL to the server at \a host and \a port. \a name is passed in the
-  query string as "title", which is used for adding a server to favorites. \a channel may be omitted, but if specified it
-  should be in the format of "/path/to/channel".
-*/
+/**
+ * This function creates a QMimeData object containing a URL to the server at host and port. name is passed in the
+ * query string as "title", which is used for adding a server to favorites. channel may be omitted, but if specified it
+ * should be in the format of "/path/to/channel".
+ */
 QMimeData *ServerItem::toMimeData(const QString &name, const QString &host, unsigned short port, const QString &channel) {
 	QUrl url;
 	url.setScheme(QLatin1String("mumble"));
@@ -596,8 +651,16 @@ QMimeData *ServerItem::toMimeData(const QString &name, const QString &host, unsi
 	if (port != DEFAULT_MUMBLE_PORT)
 		url.setPort(port);
 	url.setPath(channel);
+
+#if QT_VERSION >= 0x050000
+	QUrlQuery query;
+	query.addQueryItem(QLatin1String("title"), name);
+	query.addQueryItem(QLatin1String("version"), QLatin1String("1.2.0"));
+	url.setQuery(query);
+#else
 	url.addQueryItem(QLatin1String("title"), name);
 	url.addQueryItem(QLatin1String("version"), QLatin1String("1.2.0"));
+#endif
 
 	QString qs = QLatin1String(url.toEncoded());
 
@@ -694,22 +757,56 @@ QIcon ServerItem::loadIcon(const QString &name) {
 	return qmIcons.value(name);
 }
 
-ConnectDialogEdit::ConnectDialogEdit(QWidget *p, const QString &name, const QString &host, const QString &user, unsigned short port, const QString &password, bool add) : QDialog(p) {
+ConnectDialogEdit::ConnectDialogEdit(QWidget *p, const QString &name, const QString &host, const QString &user, unsigned short port, const QString &password) : QDialog(p) {
 	setupUi(this);
-	if (add)
-		setWindowTitle(tr("Add Server"));
+	init();
 
-	qlePort->setValidator(new QIntValidator(1, 65535, qlePort));
+	bCustomLabel = ! name.simplified().isEmpty();
 
 	qleName->setText(name);
 	qleServer->setText(host);
-	qleUsername->setText(user.isEmpty() ? g.s.qsUsername : user);
+	qleUsername->setText(user);
 	qlePort->setText(QString::number(port));
-	qlePassword->setEchoMode(QLineEdit::Password);
 	qlePassword->setText(password);
 
+	validate();
+}
+
+ConnectDialogEdit::ConnectDialogEdit(QWidget *parent) : QDialog(parent) {
+	setupUi(this);
+	setWindowTitle(tr("Add Server"));
+	init();
+
+	if (!updateFromClipboard()) {
+		// If connected to a server assume the user wants to add it
+		if (g.sh && g.sh->isRunning()) {
+			QString host, name, user, pw;
+			unsigned short port = DEFAULT_MUMBLE_PORT;
+
+			g.sh->getConnectionInfo(host, port, user, pw);
+			Channel *c = Channel::get(0);
+			if (c && c->qsName != QLatin1String("Root")) {
+				name = c->qsName;
+			}
+
+			showNotice(tr("You are currently connected to a server.\nDo you want to fill the dialog with the connection data of this server?\nHost: %1 Port: %2").arg(host).arg(port));
+			m_si = new ServerItem(name, host, port, user, pw);
+		}
+	}
+	qleUsername->setText(g.s.qsUsername);
+}
+
+void ConnectDialogEdit::init() {
+	m_si = NULL;
 	usPort = 0;
 	bOk = true;
+	bCustomLabel = false;
+
+	qwInlineNotice->hide();
+
+	qlePort->setValidator(new QIntValidator(1, 65535, qlePort));
+	qlePort->setText(QString::number(DEFAULT_MUMBLE_PORT));
+	qlePassword->setEchoMode(QLineEdit::Password);
 
 	connect(qleName, SIGNAL(textChanged(const QString &)), this, SLOT(validate()));
 	connect(qleServer, SIGNAL(textChanged(const QString &)), this, SLOT(validate()));
@@ -720,6 +817,76 @@ ConnectDialogEdit::ConnectDialogEdit(QWidget *p, const QString &name, const QStr
 	validate();
 }
 
+ConnectDialogEdit::~ConnectDialogEdit() {
+	delete m_si;
+}
+
+void ConnectDialogEdit::showNotice(const QString &text) {
+	QLabel *label = qwInlineNotice->findChild<QLabel *>(QLatin1String("qlPasteNotice"));
+	Q_ASSERT(label);
+	label->setText(text);
+	qwInlineNotice->show();
+	adjustSize();
+}
+
+bool ConnectDialogEdit::updateFromClipboard() {
+	delete m_si;
+	m_si = ServerItem::fromMimeData(QApplication::clipboard()->mimeData(), false, NULL, true);
+	bool hasServerData = m_si != NULL;
+	if (hasServerData) {
+		showNotice(tr("You have an URL in your clipboard.\nDo you want to fill the dialog with this data?\nHost: %1 Port: %2").arg(m_si->qsHostname).arg(m_si->usPort));
+		return true;
+	} else {
+		qwInlineNotice->hide();
+		adjustSize();
+		return false;
+	}
+}
+
+void ConnectDialogEdit::on_qbFill_clicked() {
+	Q_ASSERT(m_si);
+
+	qwInlineNotice->hide();
+	adjustSize();
+
+	qleName->setText(m_si->qsName);
+	qleServer->setText(m_si->qsHostname);
+	qleUsername->setText(m_si->qsUsername);
+	qlePort->setText(QString::number(m_si->usPort));
+	qlePassword->setText(m_si->qsPassword);
+
+	delete m_si;
+	m_si = NULL;
+}
+
+void ConnectDialogEdit::on_qbDiscard_clicked() {
+	qwInlineNotice->hide();
+	adjustSize();
+}
+
+void ConnectDialogEdit::on_qleName_textEdited(const QString& name) {
+	if (bCustomLabel) {
+		// If empty, then reset to automatic label.
+		// NOTE(nik@jnstw.us): You may be tempted to set qleName to qleServer, but that results in the odd
+		// UI behavior that clearing the field doesn't clear it; it'll immediately equal qleServer. Instead,
+		// leave it empty and let it update the next time qleServer updates. Code in accept will default it
+		// to qleServer if it isn't updated beforehand.
+		if (name.simplified().isEmpty()) {
+			bCustomLabel = false;
+		}
+	} else {
+		// If manually edited, set to Custom
+		bCustomLabel = true;
+	}
+}
+
+void ConnectDialogEdit::on_qleServer_textEdited(const QString& server) {
+	// If using automatic label, update it
+	if (!bCustomLabel) {
+		qleName->setText(server);
+	}
+}
+
 void ConnectDialogEdit::validate() {
 	qsName = qleName->text().simplified();
 	qsHostname = qleServer->text().simplified();
@@ -731,7 +898,7 @@ void ConnectDialogEdit::validate() {
 	qlePort->setDisabled(!qsHostname.isEmpty() && qsHostname.startsWith(QLatin1Char('@')));
 
 	// For SuperUser show password edit
-	if (qsUsername == QLatin1String("SuperUser")) {
+	if (qsUsername.toLower() == QLatin1String("superuser")) {
 		qliPassword->setVisible(true);
 		qlePassword->setVisible(true);
 		qcbShowPassword->setVisible(true);
@@ -743,14 +910,35 @@ void ConnectDialogEdit::validate() {
 		adjustSize();
 	}
 
-	bOk = ! qsName.isEmpty() && ! qsHostname.isEmpty() && ! qsUsername.isEmpty() && usPort;
+	bOk = ! qsHostname.isEmpty() && ! qsUsername.isEmpty() && usPort;
 	qdbbButtonBox->button(QDialogButtonBox::Ok)->setEnabled(bOk);
 }
 
 void ConnectDialogEdit::accept() {
 	validate();
-	if (bOk)
+	if (bOk) {
+		QString server = qleServer->text().simplified();
+
+		// If the user accidentally added a schema or path part, drop it now.
+		// We can't do so during editing as that is quite jarring.
+		const int schemaPos = server.indexOf(QLatin1String("://"));
+		if (schemaPos != -1) {
+			server.remove(0, schemaPos + 3);
+		}
+
+		const int pathPos = server.indexOf(QLatin1Char('/'));
+		if (pathPos != -1) {
+			server.resize(pathPos);
+		}
+
+		qleServer->setText(server);
+
+		if (qleName->text().simplified().isEmpty() || !bCustomLabel) {
+			qleName->setText(server);
+		}
+
 		QDialog::accept();
+	}
 }
 
 void ConnectDialogEdit::on_qcbShowPassword_toggled(bool checked) {
@@ -766,10 +954,16 @@ ConnectDialog::ConnectDialog(QWidget *p, bool autoconnect) : QDialog(p), bAutoCo
 
 	siAutoConnect = NULL;
 
+	bAllowPing = g.s.ptProxyType == Settings::NoProxy;
+	bAllowHostLookup = g.s.ptProxyType == Settings::NoProxy;
+	bAllowBonjour = g.s.ptProxyType == Settings::NoProxy;
+	bAllowFilters = g.s.ptProxyType == Settings::NoProxy;
+
 	if (tPublicServers.elapsed() >= 60 * 24 * 1000000ULL) {
 		qlPublicServers.clear();
 	}
 
+	qdbbButtonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
 	qdbbButtonBox->button(QDialogButtonBox::Ok)->setText(tr("&Connect"));
 
 	QPushButton *qpbAdd = new QPushButton(tr("&Add New..."), this);
@@ -788,11 +982,33 @@ ConnectDialog::ConnectDialog(QWidget *p, bool autoconnect) : QDialog(p), bAutoCo
 	
 	qpbAdd->setHidden(g.s.disableConnectDialogEditing);
 	qpbEdit->setHidden(g.s.disableConnectDialogEditing);
-	
+
+	qtwServers->setItemDelegate(new ServerViewDelegate());
+
+	// Hide ping and user count if we are not allowed to ping.
+	if (!bAllowPing) {
+		qtwServers->setColumnCount(1);
+	}
+
 	qtwServers->sortItems(1, Qt::AscendingOrder);
+
+#if QT_VERSION >= 0x050000
+	qtwServers->header()->setSectionResizeMode(0, QHeaderView::Stretch);
+	if (qtwServers->columnCount() >= 2) {
+		qtwServers->header()->setSectionResizeMode(1, QHeaderView::ResizeToContents);
+	}
+	if (qtwServers->columnCount() >= 3) {
+		qtwServers->header()->setSectionResizeMode(2, QHeaderView::ResizeToContents);
+	}
+#else
 	qtwServers->header()->setResizeMode(0, QHeaderView::Stretch);
-	qtwServers->header()->setResizeMode(1, QHeaderView::ResizeToContents);
-	qtwServers->header()->setResizeMode(2, QHeaderView::ResizeToContents);
+	if (qtwServers->columnCount() >= 2) {
+		qtwServers->header()->setResizeMode(1, QHeaderView::ResizeToContents);
+	}
+	if (qtwServers->columnCount() >= 3) {
+		qtwServers->header()->setResizeMode(2, QHeaderView::ResizeToContents);
+	}
+#endif
 
 	connect(qtwServers->header(), SIGNAL(sortIndicatorChanged(int, Qt::SortOrder)), this, SLOT(OnSortChanged(int, Qt::SortOrder)));
 
@@ -800,16 +1016,20 @@ ConnectDialog::ConnectDialog(QWidget *p, bool autoconnect) : QDialog(p), bAutoCo
 	qaShowReachable->setChecked(false);
 	qaShowPopulated->setChecked(false);
 
-	switch (g.s.ssFilter) {
-		case Settings::ShowPopulated:
-			qaShowPopulated->setChecked(true);
-			break;
-		case Settings::ShowReachable:
-			qaShowReachable->setChecked(true);
-			break;
-		default:
-			qaShowAll->setChecked(true);
-			break;
+	if (bAllowFilters) {
+		switch (g.s.ssFilter) {
+			case Settings::ShowPopulated:
+				qaShowPopulated->setChecked(true);
+				break;
+			case Settings::ShowReachable:
+				qaShowReachable->setChecked(true);
+				break;
+			default:
+				qaShowAll->setChecked(true);
+				break;
+		}
+	} else {
+		qaShowAll->setChecked(true);
 	}
 
 	qagFilters = new QActionGroup(this);
@@ -825,8 +1045,12 @@ ConnectDialog::ConnectDialog(QWidget *p, bool autoconnect) : QDialog(p), bAutoCo
 	qmFilters->addAction(qaShowReachable);
 	qmFilters->addAction(qaShowPopulated);
 
+	if (!bAllowFilters) {
+		qmFilters->setEnabled(false);
+	}
+
 	QList<QTreeWidgetItem *> ql;
-	QList<FavoriteServer> favorites = Database::getFavorites();
+	QList<FavoriteServer> favorites = g.db->getFavorites();
 
 	foreach(const FavoriteServer &fs, favorites) {
 		ServerItem *si = new ServerItem(fs);
@@ -837,7 +1061,7 @@ ConnectDialog::ConnectDialog(QWidget *p, bool autoconnect) : QDialog(p), bAutoCo
 
 #ifdef USE_BONJOUR
 	// Make sure the we got the objects we need, then wire them up
-	if (g.bc->bsbBrowser && g.bc->bsrResolver) {
+	if (bAllowBonjour && g.bc->bsbBrowser && g.bc->bsrResolver) {
 		connect(g.bc->bsbBrowser, SIGNAL(error(DNSServiceErrorType)),
 		        this, SLOT(onLanBrowseError(DNSServiceErrorType)));
 		connect(g.bc->bsbBrowser, SIGNAL(currentBonjourRecordsChanged(const QList<BonjourRecord> &)),
@@ -875,7 +1099,7 @@ ConnectDialog::ConnectDialog(QWidget *p, bool autoconnect) : QDialog(p), bAutoCo
 	qtwServers->setCurrentItem(NULL);
 	bLastFound = false;
 
-	qmPingCache = Database::getPingCache();
+	qmPingCache = g.db->getPingCache();
 
 	if (! g.s.qbaConnectDialogGeometry.isEmpty())
 		restoreGeometry(g.s.qbaConnectDialogGeometry);
@@ -891,14 +1115,14 @@ ConnectDialog::~ConnectDialog() {
 
 	foreach(ServerItem *si, qlItems) {
 		if (si->uiPing)
-			qmPingCache.insert(QPair<QString, unsigned short>(si->qsHostname, si->usPort), si->uiPing);
+			qmPingCache.insert(UnresolvedServerAddress(si->qsHostname, si->usPort), si->uiPing);
 
 		if (si->itType != ServerItem::FavoriteType)
 			continue;
 		ql << si->toFavoriteServer();
 	}
-	Database::setFavorites(ql);
-	Database::setPingCache(qmPingCache);
+	g.db->setFavorites(ql);
+	g.db->setPingCache(qmPingCache);
 
 	g.s.qbaConnectDialogHeader = qtwServers->header()->saveState();
 	g.s.qbaConnectDialogGeometry = saveGeometry();
@@ -906,8 +1130,8 @@ ConnectDialog::~ConnectDialog() {
 
 void ConnectDialog::accept() {
 	ServerItem *si = static_cast<ServerItem *>(qtwServers->currentItem());
-	if (! si || si->qlAddresses.isEmpty() || si->qsHostname.isEmpty()) {
-		qWarning() << "Sad panda";
+	if (! si || (bAllowHostLookup && si->qlAddresses.isEmpty()) || si->qsHostname.isEmpty()) {
+		qWarning() << "Invalid server";
 		return;
 	}
 
@@ -931,12 +1155,16 @@ void ConnectDialog::accept() {
 }
 
 void ConnectDialog::OnSortChanged(int logicalIndex, Qt::SortOrder) {
-	if (logicalIndex == 2)
-		foreach(ServerItem *si, qlItems)
-			if (si->uiPing && (si->uiPing != si->uiPingSort)) {
-				si->uiPingSort = si->uiPing;
-				si->setDatas();
-			}
+	if (logicalIndex != 2) {
+		return;
+	}
+
+	foreach(ServerItem *si, qlItems) {
+		if (si->uiPing && (si->uiPing != si->uiPingSort)) {
+			si->uiPingSort = si->uiPing;
+			si->setDatas();
+		}
+	}
 }
 
 void ConnectDialog::on_qaFavoriteAdd_triggered() {
@@ -953,39 +1181,7 @@ void ConnectDialog::on_qaFavoriteAdd_triggered() {
 }
 
 void ConnectDialog::on_qaFavoriteAddNew_triggered() {
-	QString host, user, pw;
-	QString name;
-	unsigned short port = DEFAULT_MUMBLE_PORT;
-
-	// Try to fill out fields if possible
-	{
-		ServerItem *si = ServerItem::fromMimeData(QApplication::clipboard()->mimeData());
-		if (si) {
-			// If there is server information in the clipboard assume user wants to add it
-			name = si->qsName;
-
-			if (! si->qsBonjourHost.isEmpty())
-				host = QLatin1Char('@') + si->qsBonjourHost;
-			else
-				host = si->qsHostname;
-
-			port = si->usPort;
-			pw = si->qsPassword;
-		} else {
-			// If connected to a server assume the user wants to add it
-			if (g.sh && g.sh->isRunning()) {
-				g.sh->getConnectionInfo(host, port, user, pw);
-				Channel *c = Channel::get(0);
-				if (c) {
-					if (c->qsName != QLatin1String("Root"))
-						name = c->qsName;
-				}
-			} else
-				user = g.s.qsUsername;
-		}
-	}
-
-	ConnectDialogEdit *cde = new ConnectDialogEdit(this, name, host, user, port, pw, true);
+	ConnectDialogEdit *cde = new ConnectDialogEdit(this);
 
 	if (cde->exec() == QDialog::Accepted) {
 		ServerItem *si = new ServerItem(cde->qsName, cde->qsHostname, cde->usPort, cde->qsUsername, cde->qsPassword);
@@ -1102,7 +1298,7 @@ void ConnectDialog::on_qtwServers_customContextMenuRequested(const QPoint &mpos)
 			if (si->itType == ServerItem::FavoriteType) {
 				qmPopup->addAction(qaFavoriteEdit);
 				qmPopup->addAction(qaFavoriteRemove);
-			} else if (si) {
+			} else {
 				qmPopup->addAction(qaFavoriteAdd);
 			}
 		}
@@ -1136,6 +1332,9 @@ void ConnectDialog::on_qtwServers_currentItemChanged(QTreeWidgetItem *item, QTre
 	}
 	
 	bool bOk = !si->qlAddresses.isEmpty();
+	if (!bAllowHostLookup) {
+		bOk = true;
+	}
 	qdbbButtonBox->button(QDialogButtonBox::Ok)->setEnabled(bOk);
 
 	bLastFound = true;
@@ -1160,10 +1359,16 @@ void ConnectDialog::initList() {
 	bPublicInit = true;
 
 	QUrl url;
-	url.setPath(QLatin1String("/list2.cgi"));
+	url.setPath(QLatin1String("/v1/list"));
+#if QT_VERSION >= 0x050000
+	QUrlQuery query;
+	query.addQueryItem(QLatin1String("version"), QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)));
+	url.setQuery(query);
+#else
 	url.addQueryItem(QLatin1String("version"), QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)));
+#endif
 
-	WebFetch::fetch(url, this, SLOT(fetched(QByteArray,QUrl,QMap<QString,QString>)));
+	WebFetch::fetch(QLatin1String("publist"), url, this, SLOT(fetched(QByteArray,QUrl,QMap<QString,QString>)));
 }
 
 #ifdef USE_BONJOUR
@@ -1271,22 +1476,30 @@ void ConnectDialog::timeTick() {
 				if (! siAutoConnect->qlAddresses.isEmpty()) {
 					accept();
 					return;
+				} else if (!bAllowHostLookup) {
+					accept();
+					return;
 				}
 			}
 		}
 	}
 
-	// Start DNS Lookup of first unknown hostname
-	foreach(const QString &host, qlDNSLookup) {
-		if (qsDNSActive.contains(host))
-			continue;
+	if (bAllowHostLookup) {
+		// Start DNS Lookup of first unknown hostname
+		foreach(const UnresolvedServerAddress &unresolved, qlDNSLookup) {
+			if (qsDNSActive.contains(unresolved)) {
+				continue;
+			}
 
-		qlDNSLookup.removeAll(host);
-		qlDNSLookup.append(host);
+			qlDNSLookup.removeAll(unresolved);
+			qlDNSLookup.append(unresolved);
 
-		qsDNSActive.insert(host);
-		QHostInfo::lookupHost(host, this, SLOT(lookedUp(QHostInfo)));
-		break;
+			qsDNSActive.insert(unresolved);
+			ServerResolver *sr = new ServerResolver();
+			QObject::connect(sr, SIGNAL(resolved()), this, SLOT(lookedUp()));
+			sr->resolve(unresolved.hostname, unresolved.port);
+			break;
+		}
 	}
 
 	ServerItem *current = static_cast<ServerItem *>(qtwServers->currentItem());
@@ -1300,12 +1513,14 @@ void ConnectDialog::timeTick() {
 		si = hover;
 
 	if (si) {
-		QString host = si->qsHostname.toLower();
+		QString hostname = si->qsHostname.toLower();
+		unsigned short port = si->usPort;
+		UnresolvedServerAddress unresolved(hostname, port);
 
 		if (si->qlAddresses.isEmpty()) {
-			if (! host.isEmpty()) {
-				qlDNSLookup.removeAll(host);
-				qlDNSLookup.prepend(host);
+			if (! hostname.isEmpty()) {
+				qlDNSLookup.removeAll(unresolved);
+				qlDNSLookup.prepend(unresolved);
 			}
 			si = NULL;
 		}
@@ -1341,34 +1556,46 @@ void ConnectDialog::timeTick() {
 	if (si == hover)
 		tHover.restart();
 
-	foreach(const QHostAddress &host, si->qlAddresses)
-		sendPing(host, si->usPort);
+	foreach(const ServerAddress &addr, si->qlAddresses) {
+		sendPing(addr.host.toAddress(), addr.port);
+	}
 }
 
 
 void ConnectDialog::startDns(ServerItem *si) {
-	QString host = si->qsHostname.toLower();
+	if (!bAllowHostLookup) {
+		return;
+	}
+
+	QString hostname = si->qsHostname.toLower();
+	unsigned short port = si->usPort;
+	UnresolvedServerAddress unresolved(hostname, port);
 
 	if (si->qlAddresses.isEmpty()) {
+		// Determine if qsHostname is an IP address
+		// or a hostname. If it is an IP address, we
+		// can treat it as resolved as-is.
 		QHostAddress qha(si->qsHostname);
-		if (! qha.isNull())
-			si->qlAddresses.append(qha);
-		else
-			si->qlAddresses = qhDNSCache.value(host);
+		bool hostnameIsIPAddress = !qha.isNull();
+		if (hostnameIsIPAddress) {
+			si->qlAddresses.append(ServerAddress(HostAddress(qha), port));
+		} else {
+			si->qlAddresses = qhDNSCache.value(unresolved);
+		}
 	}
 
 	if (qtwServers->currentItem() == si)
 		qdbbButtonBox->button(QDialogButtonBox::Ok)->setEnabled(! si->qlAddresses.isEmpty());
 
 	if (! si->qlAddresses.isEmpty()) {
-		foreach(const QHostAddress &qha, si->qlAddresses) {
-			qhPings[qpAddress(qha, si->usPort)].insert(si);
+		foreach(const ServerAddress &addr, si->qlAddresses) {
+			qhPings[addr].insert(si);
 		}
 		return;
 	}
 
 #ifdef USE_BONJOUR
-	if (si->qsHostname.isEmpty() && ! si->brRecord.serviceName.isEmpty()) {
+	if (bAllowBonjour && si->qsHostname.isEmpty() && ! si->brRecord.serviceName.isEmpty()) {
 		if (! qlBonjourActive.contains(si->brRecord)) {
 			g.bc->bsrResolver->resolveBonjourRecord(si->brRecord);
 			qlBonjourActive.append(si->brRecord);
@@ -1377,18 +1604,21 @@ void ConnectDialog::startDns(ServerItem *si) {
 	}
 #endif
 
-	if (! qhDNSWait.contains(host)) {
+	if (! qhDNSWait.contains(unresolved)) {
 		if (si->itType == ServerItem::PublicType)
-			qlDNSLookup.append(host);
+			qlDNSLookup.append(unresolved);
 		else
-			qlDNSLookup.prepend(host);
+			qlDNSLookup.prepend(unresolved);
 	}
-	qhDNSWait[host].insert(si);
+	qhDNSWait[unresolved].insert(si);
 }
 
 void ConnectDialog::stopDns(ServerItem *si) {
-	foreach(const QHostAddress &qha, si->qlAddresses) {
-		qpAddress addr(qha, si->usPort);
+	if (!bAllowHostLookup) {
+		return;
+	}
+
+	foreach(const ServerAddress &addr, si->qlAddresses) {
 		if (qhPings.contains(addr)) {
 			qhPings[addr].remove(si);
 			if (qhPings[addr].isEmpty()) {
@@ -1398,37 +1628,55 @@ void ConnectDialog::stopDns(ServerItem *si) {
 		}
 	}
 
-	QString host = si->qsHostname.toLower();
+	QString hostname = si->qsHostname.toLower();
+	unsigned short port = si->usPort;
+	UnresolvedServerAddress unresolved(hostname, port);
 
-	if (qhDNSWait.contains(host)) {
-		qhDNSWait[host].remove(si);
-		if (qhDNSWait[host].isEmpty()) {
-			qhDNSWait.remove(host);
-			qlDNSLookup.removeAll(host);
+	if (qhDNSWait.contains(unresolved)) {
+		qhDNSWait[unresolved].remove(si);
+		if (qhDNSWait[unresolved].isEmpty()) {
+			qhDNSWait.remove(unresolved);
+			qlDNSLookup.removeAll(unresolved);
 		}
 	}
 }
 
-void ConnectDialog::lookedUp(QHostInfo info) {
-	QString host = info.hostName().toLower();
-	qsDNSActive.remove(host);
+void ConnectDialog::lookedUp() {
+	ServerResolver *sr = qobject_cast<ServerResolver *>(QObject::sender());
+	sr->deleteLater();
 
-	if (info.error() != QHostInfo::NoError)
+	QString hostname = sr->hostname().toLower();
+	unsigned short port = sr->port();
+	UnresolvedServerAddress unresolved(hostname, port);
+
+	qsDNSActive.remove(unresolved);
+
+	// An error occurred, or no records were found.
+	if (sr->records().size() == 0) {
 		return;
+	}
 
-	qlDNSLookup.removeAll(host);
-	qhDNSCache.insert(host, info.addresses());
+	QSet<ServerAddress> qs;
+	foreach (ServerResolverRecord record, sr->records()) {
+		foreach(const HostAddress &ha, record.addresses()) {
+			qs.insert(ServerAddress(ha, record.port()));
+		}
+	}
 
-	QSet<qpAddress> qs;
-
-	foreach(ServerItem *si, qhDNSWait[host]) {
-		si->qlAddresses = info.addresses();
-		foreach(const QHostAddress &qha, info.addresses()) {
-			qpAddress addr(qha, si->usPort);
-			qs.insert(addr);
+	QSet<ServerItem *> waiting = qhDNSWait[unresolved];
+	foreach(ServerItem *si, waiting) {
+		foreach (const ServerAddress &addr, qs) {
 			qhPings[addr].insert(si);
 		}
 
+		si->qlAddresses = qs.toList();
+	}
+
+	qlDNSLookup.removeAll(unresolved);
+	qhDNSCache.insert(unresolved, qs.toList());
+	qhDNSWait.remove(unresolved);
+
+	foreach(ServerItem *si, waiting) {
 		if (si == qtwServers->currentItem()) {
 			on_qtwServers_currentItemChanged(si, si);
 			if (si == siAutoConnect)
@@ -1436,17 +1684,17 @@ void ConnectDialog::lookedUp(QHostInfo info) {
 		}
 	}
 
-	qhDNSWait.remove(host);
-
-	foreach(const qpAddress &addr, qs) {
-		sendPing(addr.first, addr.second);
+	if (bAllowPing) {
+		foreach(const ServerAddress &addr, qs) {
+			sendPing(addr.host.toAddress(), addr.port);
+		}
 	}
 }
 
 void ConnectDialog::sendPing(const QHostAddress &host, unsigned short port) {
 	char blob[16];
 
-	qpAddress addr(host, port);
+	ServerAddress addr(HostAddress(host), port);
 
 	quint64 uiRand;
 	if (qhPingRand.contains(addr)) {
@@ -1474,6 +1722,7 @@ void ConnectDialog::sendPing(const QHostAddress &host, unsigned short port) {
 
 void ConnectDialog::udpReply() {
 	QUdpSocket *sock = qobject_cast<QUdpSocket *>(sender());
+
 	while (sock->hasPendingDatagrams()) {
 		char blob[64];
 
@@ -1485,7 +1734,8 @@ void ConnectDialog::udpReply() {
 			if (host.scopeId() == QLatin1String("0"))
 				host.setScopeId(QLatin1String(""));
 
-			qpAddress address(host, port);
+			ServerAddress address(HostAddress(host), port);
+
 			if (qhPings.contains(address)) {
 				quint32 *ping = reinterpret_cast<quint32 *>(blob+4);
 				quint64 *ts = reinterpret_cast<quint64 *>(blob+8);
@@ -1499,7 +1749,7 @@ void ConnectDialog::udpReply() {
 					si->uiBandwidth = qFromBigEndian(ping[5]);
 
 					if (! si->uiPingSort)
-						si->uiPingSort = qmPingCache.value(QPair<QString, unsigned short>(si->qsHostname, si->usPort));
+						si->uiPingSort = qmPingCache.value(UnresolvedServerAddress(si->qsHostname, si->usPort));
 
 					si->setDatas(static_cast<double>(elapsed), users, maxusers);
 					si->hideCheck();
@@ -1509,14 +1759,14 @@ void ConnectDialog::udpReply() {
 	}
 }
 
-void ConnectDialog::fetched(QByteArray data, QUrl, QMap<QString, QString> headers) {
-	if (data.isNull()) {
+void ConnectDialog::fetched(QByteArray xmlData, QUrl, QMap<QString, QString> headers) {
+	if (xmlData.isNull()) {
 		QMessageBox::warning(this, QLatin1String("Mumble"), tr("Failed to fetch server list"), QMessageBox::Ok);
 		return;
 	}
 
 	QDomDocument doc;
-	doc.setContent(data);
+	doc.setContent(xmlData);
 
 	qlPublicServers.clear();
 	qsUserCountry = headers.value(QLatin1String("Geo-Country"));
@@ -1534,7 +1784,7 @@ void ConnectDialog::fetched(QByteArray data, QUrl, QMap<QString, QString> header
 				pi.quUrl = e.attribute(QLatin1String("url"));
 				pi.qsIp = e.attribute(QLatin1String("ip"));
 				pi.usPort = e.attribute(QLatin1String("port")).toUShort();
-				pi.qsCountry = e.attribute(QLatin1String("country"));
+				pi.qsCountry = e.attribute(QLatin1String("country"), tr("Unknown"));
 				pi.qsCountryCode = e.attribute(QLatin1String("country_code")).toLower();
 				pi.qsContinentCode = e.attribute(QLatin1String("continent_code")).toLower();
 				pi.bCA = e.attribute(QLatin1String("ca")).toInt() ? true : false;
