@@ -1,33 +1,7 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
@@ -41,7 +15,6 @@
 #include "Connection.h"
 #include "ConnectDialog.h"
 #include "Database.h"
-#include "Global.h"
 #include "GlobalShortcut.h"
 #include "Log.h"
 #include "MainWindow.h"
@@ -54,6 +27,10 @@
 #include "UserModel.h"
 #include "VersionCheck.h"
 #include "ViewCert.h"
+#include "CryptState.h"
+
+// We define a global macro called 'g'. This can lead to issues when included code uses 'g' as a type or parameter name (like protobuf 3.7 does). As such, for now, we have to make this our last include.
+#include "Global.h"
 
 #define ACTOR_INIT \
 	ClientUser *pSrc=NULL; \
@@ -63,14 +40,17 @@
 
 #define VICTIM_INIT \
 	ClientUser *pDst=ClientUser::get(msg.session()); \
-	 if (! pDst) { \
- 		qWarning("MainWindow: Message for nonexistant victim %d.", msg.session()); \
+	 if (!pDst) { \
+ 		qWarning("MainWindow: Message for nonexistent victim %d.", msg.session()); \
 		return; \
 	}
 
 #define SELF_INIT \
-	ClientUser *pSelf = ClientUser::get(g.uiSession);
-
+	ClientUser *pSelf = ClientUser::get(g.uiSession); \
+	if (!pSelf) { \
+		qWarning("MainWindow: Received message outside of session (sid %d).", g.uiSession); \
+		return; \
+	}
 
 void MainWindow::msgAuthenticate(const MumbleProto::Authenticate &) {
 }
@@ -103,6 +83,9 @@ void MainWindow::msgReject(const MumbleProto::Reject &msg) {
 		case MumbleProto::Reject_RejectType_WrongServerPW:
 			reason = tr("Wrong password");
 			break;
+		case MumbleProto::Reject_RejectType_AuthenticatorFail:
+			reason = tr("Your account information can not be verified currently. Please try again later");
+			break;
 		default:
 			reason = Qt::escape(u8(msg.reason()));
 			break;
@@ -116,9 +99,14 @@ void MainWindow::msgServerSync(const MumbleProto::ServerSync &msg) {
 	g.sh->sendPing(); // Send initial ping to establish UDP connection
 
 	g.uiSession = msg.session();
-	g.pPermissions = static_cast<ChanACL::Permissions>(msg.permissions());
+	g.pPermissions = ChanACL::Permissions(static_cast<unsigned int>(msg.permissions()));
 	g.l->clearIgnore();
-	g.l->log(Log::Information, tr("Welcome message: %1").arg(u8(msg.welcome_text())));
+	if (msg.has_welcome_text()) {
+		QString str = u8(msg.welcome_text());
+		if (!str.isEmpty()) {
+			g.l->log(Log::Information, tr("Welcome message: %1").arg(str));
+		}
+	}
 	pmModel->ensureSelfVisible();
 	pmModel->recheckLinks();
 
@@ -138,7 +126,7 @@ void MainWindow::msgServerSync(const MumbleProto::ServerSync &msg) {
 
 	g.sh->getConnectionInfo(host, port, uname, pw);
 
-	QList<Shortcut> sc = Database::getShortcuts(g.sh->qbaDigest);
+	QList<Shortcut> sc = g.db->getShortcuts(g.sh->qbaDigest);
 	if (! sc.isEmpty()) {
 		for (int i=0;i<sc.count(); ++i) {
 			Shortcut &s = sc[i];
@@ -148,9 +136,12 @@ void MainWindow::msgServerSync(const MumbleProto::ServerSync &msg) {
 		GlobalShortcutEngine::engine->bNeedRemap = true;
 	}
 
-	ClientUser *p=ClientUser::get(g.uiSession);
-	connect(p, SIGNAL(talkingChanged()), this, SLOT(talkingChanged()));
-
+	const ClientUser *user = ClientUser::get(g.uiSession);
+	connect(user, SIGNAL(talkingStateChanged()), this, SLOT(userStateChanged()));
+	connect(user, SIGNAL(muteDeafStateChanged()), this, SLOT(userStateChanged()));
+	connect(user, SIGNAL(prioritySpeakerStateChanged()), this, SLOT(userStateChanged()));
+	connect(user, SIGNAL(recordingStateChanged()), this, SLOT(userStateChanged()));
+	
 	qstiIcon->setToolTip(tr("Mumble: %1").arg(Qt::escape(Channel::get(0)->qsName)));
 
 	// Update QActions and menues
@@ -165,8 +156,12 @@ void MainWindow::msgServerSync(const MumbleProto::ServerSync &msg) {
 
 
 void MainWindow::msgServerConfig(const MumbleProto::ServerConfig &msg) {
-	if (msg.has_welcome_text())
-		g.l->log(Log::Information, tr("Welcome message: %1").arg(u8(msg.welcome_text())));
+	if (msg.has_welcome_text()) {
+		QString str = u8(msg.welcome_text());
+		if (!str.isEmpty()) {
+			g.l->log(Log::Information, tr("Welcome message: %1").arg(str));
+		}
+	}
 	if (msg.has_max_bandwidth())
 		AudioInput::setMaxBandwidth(msg.max_bandwidth());
 	if (msg.has_allow_html())
@@ -175,6 +170,8 @@ void MainWindow::msgServerConfig(const MumbleProto::ServerConfig &msg) {
 		g.uiMessageLength = msg.message_length();
 	if (msg.has_image_message_length())
 		g.uiImageLength = msg.image_message_length();
+	if (msg.has_max_users())
+		g.uiMaxUsers = msg.max_users();
 }
 
 void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
@@ -206,21 +203,18 @@ void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
 			break;
 		case MumbleProto::PermissionDenied_DenyType_H9K: {
 				if (g.bHappyEaster) {
-					unsigned short p;
-					QString h, u, pw;
 					bool bold = g.s.bDeaf;
 					bool bold2 = g.s.bTTS;
-					g.sh->getConnectionInfo(h, p, u, pw);
 					g.s.bDeaf = false;
 					g.s.bTTS = true;
 					quint32 oflags = g.s.qmMessages.value(Log::PermissionDenied);
 					g.s.qmMessages[Log::PermissionDenied] = (oflags | Settings::LogTTS) & (~Settings::LogSoundfile);
-					g.l->log(Log::PermissionDenied, QString::fromAscii(g.ccHappyEaster + 39).arg(Qt::escape(u)));
+					g.l->log(Log::PermissionDenied, QString::fromUtf8(g.ccHappyEaster + 39).arg(Qt::escape(g.s.qsUsername)));
 					g.s.qmMessages[Log::PermissionDenied] = oflags;
 					g.s.bDeaf = bold;
 					g.s.bTTS = bold2;
-					g.mw->setWindowIcon(QIcon(QLatin1String(g.ccHappyEaster)));
-					g.mw->setStyleSheet(QString::fromAscii(g.ccHappyEaster + 82));
+					g.mw->setWindowIcon(QIcon(QString::fromUtf8(g.ccHappyEaster)));
+					g.mw->setStyleSheet(QString::fromUtf8(g.ccHappyEaster + 82));
 					qWarning() << "Happy Easter";
 				}
 			}
@@ -253,6 +247,10 @@ void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
 				g.l->log(Log::PermissionDenied, tr("Channel nesting limit reached."));
 			}
 			break;
+		case MumbleProto::PermissionDenied_DenyType_ChannelCountLimit: {
+				g.l->log(Log::PermissionDenied, tr("Channel count limit reached. Need to delete channels before creating new ones."));
+			}
+			break;
 		default: {
 				if (msg.has_reason())
 					g.l->log(Log::PermissionDenied, tr("Denied: %1.").arg(Qt::escape(u8(msg.reason()))));
@@ -268,35 +266,118 @@ void MainWindow::msgUDPTunnel(const MumbleProto::UDPTunnel &) {
 
 void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 	ACTOR_INIT;
-	SELF_INIT;
+	ClientUser* pSelf = ClientUser::get(g.uiSession);
 	ClientUser *pDst = ClientUser::get(msg.session());
-	bool bNewUser = false;
+	Channel *channel = NULL;
 
-	if (! pDst) {
-		if (msg.has_name()) {
-			pDst = pmModel->addUser(msg.session(), u8(msg.name()));
-			bNewUser = true;
-		} else {
-			return;
+	if (msg.has_channel_id()) {
+		channel = Channel::get(msg.channel_id());
+		if (!channel) {
+			qWarning("msgUserState(): unknown channel.");
+			channel = Channel::get(0);
 		}
 	}
 
-	if (msg.has_user_id())
-		pmModel->setUserId(pDst, msg.user_id());
+	// User just connected
+	if (!pDst) {
+		if (!msg.has_name()) {
+			return;
+		}
 
-	if (msg.has_hash()) {
-		pmModel->setHash(pDst, u8(msg.hash()));
-		const QString &name = Database::getFriend(pDst->qsHash);
-		if (! name.isEmpty())
-			pmModel->setFriendName(pDst, name);
-		if (Database::isLocalMuted(pDst->qsHash))
-			pDst->setLocalMute(true);
-		if (Database::isLocalIgnored(pDst->qsHash))
-			pDst->setLocalIgnore(true);
+		pDst = pmModel->addUser(msg.session(), u8(msg.name()));
+
+		if (channel) {
+			pmModel->moveUser(pDst, channel);
+		}
+
+		if (msg.has_hash()) {
+			pmModel->setHash(pDst, u8(msg.hash()));
+		}
+
+		if (pSelf) {
+			if (pDst->cChannel == pSelf->cChannel) {
+				g.l->log(Log::ChannelJoinConnect, tr("%1 connected and entered channel.").arg(Log::formatClientUser(pDst, Log::Source)));
+			} else {
+				g.l->log(Log::UserJoin, tr("%1 connected.").arg(Log::formatClientUser(pDst, Log::Source)));
+			}
+		}
 	}
 
-	if (bNewUser)
-		g.l->log(Log::UserJoin, tr("%1 connected.").arg(Log::formatClientUser(pDst, Log::Source)));
+	if (msg.has_user_id()) {
+		pmModel->setUserId(pDst, msg.user_id());
+	}
+
+	if (channel) {
+		Channel *oldChannel = pDst->cChannel;
+		if (channel != oldChannel) {
+			pmModel->moveUser(pDst, channel);
+
+			if (pSelf) {
+				if (pDst == pSelf) {
+					g.mw->updateChatBar();
+					qsDesiredChannel = channel->getPath();
+				}
+
+				if (pDst == pSelf) {
+					if (pSrc == pSelf) {
+						g.l->log(Log::SelfChannelJoin, tr("You joined %1.").arg(Log::formatChannel(channel)));
+					} else {
+						g.l->log(Log::SelfChannelJoinOther, tr("You were moved to %1 by %2.").arg(Log::formatChannel(channel)).arg(Log::formatClientUser(pSrc, Log::Source)));
+					}
+				} else if (pSrc == pSelf) {
+					if (channel == pSelf->cChannel) {
+						g.l->log(Log::ChannelJoin, tr("You moved %1 to %2.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(channel)));
+					} else {
+						g.l->log(Log::ChannelLeave, tr("You moved %1 to %2.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(channel)));
+					}
+				} else if ((channel == pSelf->cChannel) || (oldChannel == pSelf->cChannel)) {
+					if (pDst == pSrc) {
+						if (channel == pSelf->cChannel) {
+							g.l->log(Log::ChannelJoin, tr("%1 entered channel.").arg(Log::formatClientUser(pDst, Log::Target)));
+						} else {
+							g.l->log(Log::ChannelLeave, tr("%1 moved to %2.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(channel)));
+						}
+					} else {
+						if (channel == pSelf->cChannel) {
+							g.l->log(Log::ChannelJoin, tr("%1 moved in from %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(oldChannel)).arg(Log::formatClientUser(pSrc, Log::Source)));
+						} else {
+							g.l->log(Log::ChannelLeave, tr("%1 moved to %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(channel)).arg(Log::formatClientUser(pSrc, Log::Source)));
+						}
+					}
+				}
+
+				if ((channel == pSelf->cChannel) && pDst->bRecording) {
+					g.l->log(Log::Recording, tr("%1 is recording").arg(Log::formatClientUser(pDst, Log::Target)));
+				}
+			}
+		}
+	}
+
+	if (msg.has_name()) {
+		QString oldName = pDst->qsName;
+		QString newName = u8(msg.name());
+		pmModel->renameUser(pDst, newName);
+		if (! oldName.isNull() && oldName != newName) {
+			if (pSrc != pDst) {
+				g.l->log(Log::UserRenamed, tr("%1 renamed to %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target, oldName))
+					.arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatClientUser(pSrc, Log::Source)));
+			} else {
+				g.l->log(Log::UserRenamed, tr("%1 renamed to %2.").arg(Log::formatClientUser(pDst, Log::Target, oldName),
+					Log::formatClientUser(pDst, Log::Target)));
+			}
+		}
+	}
+
+	if (!pDst->qsHash.isEmpty()) {
+		const QString &name = g.db->getFriend(pDst->qsHash);
+		if (! name.isEmpty())
+			pmModel->setFriendName(pDst, name);
+		if (g.db->isLocalMuted(pDst->qsHash))
+			pDst->setLocalMute(true);
+		if (g.db->isLocalIgnored(pDst->qsHash))
+			pDst->setLocalIgnore(true);
+		pDst->fLocalVolume = g.db->getUserLocalVolume(pDst->qsHash);
+	}
 
 	if (msg.has_self_deaf() || msg.has_self_mute()) {
 		if (msg.has_self_mute())
@@ -304,8 +385,7 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 		if (msg.has_self_deaf())
 			pDst->setSelfDeaf(msg.self_deaf());
 
-		if (pSelf && pDst != pSelf && (pDst->cChannel == pSelf->cChannel)) {
-			QString name = pDst->qsName;
+		if (pSelf && pDst != pSelf && ((pDst->cChannel == pSelf->cChannel) || pDst->cChannel->allLinks().contains(pSelf->cChannel))) {
 			if (pDst->bSelfMute && pDst->bSelfDeaf)
 				g.l->log(Log::OtherSelfMute, tr("%1 is now muted and deafened.").arg(Log::formatClientUser(pDst, Log::Target)));
 			else if (pDst->bSelfMute)
@@ -336,17 +416,53 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 		}
 	}
 
-	if (msg.has_deaf() || msg.has_mute() || msg.has_suppress() || msg.has_priority_speaker()) {
+	if (msg.has_priority_speaker()) {
+		if (pSelf && ((pDst->cChannel == pSelf->cChannel) || (pDst->cChannel->allLinks().contains(pSelf->cChannel)) || (pSrc == pSelf))) {
+			if ((pSrc == pDst) && (pSrc == pSelf)) {
+				if (pDst->bPrioritySpeaker) {
+					g.l->log(Log::YouMuted, tr("You revoked your priority speaker status."));
+				} else  {
+					g.l->log(Log::YouMuted, tr("You assumed priority speaker status."));
+				}
+			} else if ((pSrc != pSelf) && (pDst == pSelf) ) {
+				if (pDst->bPrioritySpeaker) {
+					g.l->log(Log::YouMutedOther, tr("%1 revoked your priority speaker status.").arg(Log::formatClientUser(pSrc, Log::Source)));
+				} else  {
+					g.l->log(Log::YouMutedOther, tr("%1 gave you priority speaker status.").arg(Log::formatClientUser(pSrc, Log::Source)));
+				}
+			} else if ((pSrc == pSelf) && (pSrc != pDst)) {
+				if (pDst->bPrioritySpeaker) {
+					g.l->log(Log::YouMutedOther, tr("You revoked priority speaker status for %1.").arg(Log::formatClientUser(pDst, Log::Target)));
+				} else  {
+					g.l->log(Log::YouMutedOther, tr("You gave priority speaker status to %1.").arg(Log::formatClientUser(pDst, Log::Target)));
+				}
+			} else if ((pSrc == pDst) && (pSrc != pSelf)) {
+				if (pDst->bPrioritySpeaker) {
+					g.l->log(Log::OtherMutedOther, tr("%1 revoked own priority speaker status.").arg(Log::formatClientUser(pSrc, Log::Source)));
+				} else {
+					g.l->log(Log::OtherMutedOther, tr("%1 assumed priority speaker status.").arg(Log::formatClientUser(pSrc, Log::Source)));
+				}
+			} else if ((pSrc != pSelf) && (pDst != pSelf)) {
+				if (pDst->bPrioritySpeaker) {
+					g.l->log(Log::OtherMutedOther, tr("%1 revoked priority speaker status for %2.").arg(Log::formatClientUser(pSrc, Log::Source), Log::formatClientUser(pDst, Log::Target)));
+				} else if (!pDst->bPrioritySpeaker) {
+					g.l->log(Log::OtherMutedOther, tr("%1 gave priority speaker status to %2.").arg(Log::formatClientUser(pSrc, Log::Source), Log::formatClientUser(pDst, Log::Target)));
+				}
+			}
+		}
+
+		pDst->setPrioritySpeaker(msg.priority_speaker());
+	}
+
+	if (msg.has_deaf() || msg.has_mute() || msg.has_suppress()) {
 		if (msg.has_mute())
 			pDst->setMute(msg.mute());
 		if (msg.has_deaf())
 			pDst->setDeaf(msg.deaf());
 		if (msg.has_suppress())
 			pDst->setSuppress(msg.suppress());
-		if (msg.has_priority_speaker())
-			pDst->setPrioritySpeaker(msg.priority_speaker());
 
-		if (pSelf && ((pDst->cChannel == pSelf->cChannel) || (pSrc == pSelf))) {
+		if (pSelf && ((pDst->cChannel == pSelf->cChannel) || (pDst->cChannel->allLinks().contains(pSelf->cChannel)) || (pSrc == pSelf))) {
 			if (pDst == pSelf) {
 				if (msg.has_mute() && msg.has_deaf() && pDst->bMute && pDst->bDeaf) {
 					g.l->log(Log::YouMuted, tr("You were muted and deafened by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
@@ -436,49 +552,6 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 		}
 	}
 
-	if (msg.has_channel_id()) {
-		Channel *c = Channel::get(msg.channel_id());
-		if (!c) {
-			qWarning("MessageUserMove for unknown channel.");
-			c = Channel::get(0);
-		}
-
-		Channel *old = pDst->cChannel;
-		if (c != old) {
-			bool log = pSelf && !((pDst == pSelf) && (pSrc == pSelf));
-
-			if (log) {
-				if (pDst == pSelf) {
-					g.l->log(Log::ChannelJoin, tr("You were moved to %1 by %2.").arg(Log::formatChannel(c)).arg(Log::formatClientUser(pSrc, Log::Source)));
-				} else if (pDst->cChannel == pSelf->cChannel) {
-					if (pDst == pSrc)
-						g.l->log(Log::ChannelLeave, tr("%1 moved to %2.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(c)));
-					else
-						g.l->log(Log::ChannelLeave, tr("%1 moved to %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(c)).arg(Log::formatClientUser(pSrc, Log::Source)));
-				}
-			}
-
-			pmModel->moveUser(pDst, c);
-
-			if (pDst == pSelf) {
-				g.mw->updateChatBar();
-				qsDesiredChannel = c->getPath();
-			}
-
-			if (log && (pDst != pSelf) && (pDst->cChannel == pSelf->cChannel)) {
-				if (pDst == pSrc)
-					g.l->log(Log::ChannelJoin, tr("%1 entered channel.").arg(Log::formatClientUser(pDst, Log::Target)));
-				else
-					g.l->log(Log::ChannelJoin, tr("%1 moved in from %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(old)).arg(Log::formatClientUser(pSrc, Log::Source)));
-
-				if (pDst->bRecording)
-					g.l->log(Log::Recording, tr("%1 is recording").arg(Log::formatClientUser(pDst, Log::Target)));
-			}
-		}
-	}
-	if (msg.has_name()) {
-		pmModel->renameUser(pDst, u8(msg.name()));
-	}
 	if (msg.has_texture_hash()) {
 		pDst->qbaTextureHash = blob(msg.texture_hash());
 		pDst->qbaTexture = QByteArray();
@@ -490,7 +563,7 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 			pDst->qbaTextureHash = QByteArray();
 		} else {
 			pDst->qbaTextureHash = sha1(pDst->qbaTexture);
-			Database::setBlob(pDst->qbaTextureHash, pDst->qbaTexture);
+			g.db->setBlob(pDst->qbaTextureHash, pDst->qbaTexture);
 		}
 		g.o->verifyTexture(pDst);
 	}
@@ -519,7 +592,11 @@ void MainWindow::msgUserRemove(const MumbleProto::UserRemove &msg) {
 		else
 			g.l->log((pSrc == pSelf) ? Log::YouKicked : Log::UserKicked, tr("%3 was kicked from the server by %1: %2.").arg(Log::formatClientUser(pSrc, Log::Source)).arg(reason).arg(Log::formatClientUser(pDst, Log::Target)));
 	} else {
-		g.l->log(Log::UserLeave, tr("%1 disconnected.").arg(Log::formatClientUser(pDst, Log::Source)));
+		if (pDst->cChannel == pSelf->cChannel || pDst->cChannel->allLinks().contains(pSelf->cChannel)) {
+			g.l->log(Log::ChannelLeaveDisconnect, tr("%1 left channel and disconnected.").arg(Log::formatClientUser(pDst, Log::Source)));
+		} else {
+			g.l->log(Log::UserLeave, tr("%1 disconnected.").arg(Log::formatClientUser(pDst, Log::Source)));
+		}
 	}
 	if (pDst != pSelf)
 		pmModel->removeUser(pDst);
@@ -533,20 +610,31 @@ void MainWindow::msgChannelState(const MumbleProto::ChannelState &msg) {
 	Channel *p = msg.has_parent() ? Channel::get(msg.parent()) : NULL;
 
 	if (!c) {
-		if (msg.has_parent() && p && msg.has_name()) {
+		// Addresses channel does not exist so create it
+		if (p && msg.has_name()) {
 			c = pmModel->addChannel(msg.channel_id(), p, u8(msg.name()));
 			c->bTemporary = msg.temporary();
-			p = NULL;
+			p = NULL; // No need to move it later
+
+			ServerHandlerPtr sh = g.sh;
+			if (sh)
+				c->bFiltered = g.db->isChannelFiltered(sh->qbaDigest, c->iId);
+
 		} else {
+			qWarning("Server attempted state change on nonexistent channel");
 			return;
 		}
 	}
 
 	if (p) {
+		// Channel move
 		Channel *pp = p;
 		while (pp) {
-			if (pp == c)
+			if (pp == c) {
+				qWarning("Server asked to move a channel into itself or one of its children");
 				return;
+			}
+
 			pp = pp->cParent;
 		}
 		pmModel->moveChannel(c, p);
@@ -595,12 +683,27 @@ void MainWindow::msgChannelState(const MumbleProto::ChannelState &msg) {
 		if (! ql.isEmpty())
 			pmModel->linkChannels(c, ql);
 	}
+
+	if (msg.has_max_users()) {
+		c->uiMaxUsers = msg.max_users();
+	}
 }
 
 void MainWindow::msgChannelRemove(const MumbleProto::ChannelRemove &msg) {
 	Channel *c = Channel::get(msg.channel_id());
-	if (c && (c->iId != 0))
-		pmModel->removeChannel(c);
+	if (c && (c->iId != 0)) {
+		if (c->bFiltered) {
+			ServerHandlerPtr sh = g.sh;
+			if (sh)
+				g.db->setChannelFiltered(sh->qbaDigest, c->iId, false);
+			c->bFiltered = false;
+		}
+		if (!pmModel->removeChannel(c, true)) {
+			g.l->log(Log::CriticalError, tr("Protocol violation. Server sent remove for occupied channel."));
+			g.sh->disconnect();
+			return;
+		}
+	}
 }
 
 void MainWindow::msgTextMessage(const MumbleProto::TextMessage &msg) {
@@ -613,14 +716,19 @@ void MainWindow::msgTextMessage(const MumbleProto::TextMessage &msg) {
 
 	const QString &plainName = pSrc ? pSrc->qsName : tr("Server", "message from");
 	const QString &name = pSrc ? Log::formatClientUser(pSrc, Log::Source) : tr("Server", "message from");
+	bool privateMessage = false;
 
 	if (msg.tree_id_size() > 0) {
 		target += tr("(Tree) ");
 	} else if (msg.channel_id_size() > 0) {
 		target += tr("(Channel) ");
+	} else if (msg.session_size() > 0) {
+		target += tr("(Private) ");
+		privateMessage = true;
 	}
 
-	g.l->log(Log::TextMessage, tr("%2%1: %3").arg(name).arg(target).arg(u8(msg.message())),
+	g.l->log(privateMessage ? Log::PrivateTextMessage : Log::TextMessage,
+	         tr("%2%1: %3").arg(name).arg(target).arg(u8(msg.message())),
 	         tr("Message from %1").arg(plainName));
 }
 
@@ -652,7 +760,7 @@ void MainWindow::msgCryptSetup(const MumbleProto::CryptSetup &msg) {
 		const std::string &key = msg.key();
 		const std::string &client_nonce = msg.client_nonce();
 		const std::string &server_nonce = msg.server_nonce();
-		if (key.size() == AES_BLOCK_SIZE && client_nonce.size() == AES_BLOCK_SIZE && server_nonce.size() == AES_BLOCK_SIZE)
+		if (key.size() == AES_KEY_SIZE_BYTES && client_nonce.size() == AES_BLOCK_SIZE && server_nonce.size() == AES_BLOCK_SIZE)
 			c->csCrypt.setKey(reinterpret_cast<const unsigned char *>(key.data()), reinterpret_cast<const unsigned char *>(client_nonce.data()), reinterpret_cast<const unsigned char *>(server_nonce.data()));
 	} else if (msg.has_server_nonce()) {
 		const std::string &server_nonce = msg.server_nonce();
@@ -765,7 +873,13 @@ void MainWindow::msgCodecVersion(const MumbleProto::CodecVersion &msg) {
 	bool pref = msg.prefer_alpha();
 
 #ifdef USE_OPUS
+	static bool warnedOpus = false;
 	g.bOpus = msg.opus();
+
+	if (!g.oCodec && !warnedOpus) {
+		g.l->log(Log::CriticalError, tr("Failed to load Opus, it will not be available for audio encoding/decoding."));
+		warnedOpus = true;
+	}
 #endif
 
 	// Workaround for broken 1.2.2 servers
@@ -790,15 +904,15 @@ void MainWindow::msgCodecVersion(const MumbleProto::CodecVersion &msg) {
 
 	int willuse = pref ? g.iCodecAlpha : g.iCodecBeta;
 
-	static bool warned = false;
+	static bool warnedCELT = false;
 
 	if (! g.qmCodecs.contains(willuse)) {
-		if (! warned) {
+		if (! warnedCELT) {
 			g.l->log(Log::CriticalError, tr("Unable to find matching CELT codecs with other clients. You will not be able to talk to all users."));
-			warned = true;
+			warnedCELT = true;
 		}
 	} else {
-		warned = false;
+		warnedCELT = false;
 	}
 }
 
